//
//  OCRSDK.swift
//  OCR-SDK
//
//  Created by itsol on 5/18/20.
//  Copyright © 2020 itsol. All rights reserved.
//

import Foundation
import UIKit

public class SB_KYC_SDK {
    public static let shared = SB_KYC_SDK()
    public typealias CompletionHandle = (_ data: [String: Any]?, _ cardImage: Data?, _ faceImage: Data?) -> Void
    public typealias CompletionHandleMatch = (_ data: [String: Any]?) -> Void
    public typealias CompletionHandleFace = (_ data: Bool) -> Void
    let viewController = SBKTutorialVC(URLToken: "http://sdk.sb.gotai.ml")
    
    //Khởi tạo framework
    //input: viewCurrent, sdkConfig
    public func initSDK(viewCurrent: UIViewController, sdkConfig: [String:Any], completion: @escaping CompletionHandle) {
        DispatchQueue.main.async {
            Global.language = sdkConfig["language"] as? String ?? Global.language
            Global.url_font = sdkConfig["url_font"] as? String ?? Global.url_font
            Global.url_back = sdkConfig["url_back"] as? String ?? Global.url_back
            Global.url_face = sdkConfig["url_face"] as? String ?? Global.url_face
            Global.header = sdkConfig["header"] as? [String: Any] ?? Global.header
            Global.typeFace = sdkConfig["type_face"] as? String ?? Global.typeFace
            Global.colorConfig = sdkConfig["color"] as? String ?? Global.colorConfig
            Global.frontConfig = sdkConfig["front"] as? String ?? Global.frontConfig
            Global.copyright = sdkConfig["copyright"] as? String ?? Global.copyright
            Global.step = sdkConfig["step"] as? String ?? Global.step
            Global.colorButtonNext = sdkConfig["colorBackgroundButtonNext"] as? String ?? Global.colorButtonNext
            Global.colorButtonBack = sdkConfig["colorBackgroundButtonBack"] as? String ?? Global.colorButtonBack
            Global.colorTextButtonNext = sdkConfig["colorTextButtonNext"] as? String ?? Global.colorTextButtonNext
            Global.colorTextButtonBack = sdkConfig["colorTextButtonBack"] as? String ?? Global.colorTextButtonBack
            Global.colorTextPreview = sdkConfig["colorTextPreview"] as? String ?? Global.colorTextPreview
            let ratio = sdkConfig["ratioPass"] as? Int ?? 70
            Global.ratioPass = Float(ratio)
            Global.useFileCer = sdkConfig[""] as? Bool ?? Global.useFileCer
            
            self.viewController.completionSuccessTutorial = { [weak self] data, cardImage, faceImage in
                guard self != nil else { return }
                completion(data, cardImage, faceImage)
            }
            viewCurrent.navigationController?.pushViewController(self.viewController, animated: true)
            viewCurrent.navigationController?.setNavigationBarHidden(false, animated: false)
        }
    }
    
    //Validate card
    //input: UIImage
    //output: Int, 0: Image fake, 1: Image front, 2: Image back, -1: error
    public func eKycCardValid(image: Data) -> ValidateCard {
        let dataImage = UIImage(data: image)
        let imageCVPB = SBKValidateInput.shared.comvertUIImageToCVPixel(imageInput: dataImage!)
        let resultVaidate = SBKValidateInput.shared.didOutput(pixelBuffer: imageCVPB)
        
        return resultVaidate
    }
    
    //Validate face
    //input: UIImage
    public func eKycFaceValid(image: Data, completion: @escaping CompletionHandleFace) {
        let dataImage = UIImage(data: image)
        let imageCVPB = SBKValidateInput.shared.comvertUIImageToCVPixel(imageInput: dataImage!)
        SBKValidateInput.shared.validateFace(imageFace: imageCVPB) {[weak self] data in
            guard self != nil else { return }
            completion(data)
        }
    }
    
    //Call API face
    //input: UIImage
    public func eKycFaceMatch(imageFace: Data, imageCardFront: Data, imageCardBack: Data, sdkConfig: [String:Any], completion: @escaping CompletionHandleMatch) {
        Global.imageCard1 = imageCardFront
        Global.imageCard2 = imageCardBack
        Global.language = sdkConfig["language"] as? String ?? Global.language
        Global.url_face = sdkConfig["url_match"] as? String ?? Global.url_face
        
        SBOCRRequest.shared.processFace(image: imageFace , pathURL: Global.url_face, idBack: "", idFront: "") {(errorCode,data) -> Void in
            guard errorCode == nil else {
                let error: [String: Any] = ["error": errorCode!]
                completion(error)
                return
            }
            completion(data)
        }
    }
}
