//
//  CaptureCardVC.swift
//
//  Created by itsol on 5/11/20.
//  Copyright © 2020 itsol. All rights reserved.
//

import UIKit
import AVFoundation

class SBKCaptureCardVC: UIViewController, AVCapturePhotoCaptureDelegate {
    @IBOutlet public weak var viewCamera: UIView!
    @IBOutlet public weak var lbDescription: UILabel!
    @IBOutlet weak var lbCopyright: UILabel!
    @IBOutlet weak var btnRotateScreen: UIButton!
    @IBOutlet weak var lbStep: UILabel!
    @IBOutlet weak var btnCapture: UIButton!
    @IBOutlet weak var imgCaution: UIImageView!
    
    var captureSession: AVCaptureSession = AVCaptureSession()
    var stillImageOutput: AVCapturePhotoOutput =  AVCapturePhotoOutput()
    var videoPreviewLayer: AVCaptureVideoPreviewLayer!
    let videoDataOutput = AVCaptureVideoDataOutput()
    
    var descriptionScreen: String = "Front of your personal card".localized()
    var checkScreen: Int = 1 // checkScreen = 1: screen front card, checkScreen = 2: screen front
    var idFront: String = ""
    var URLToken: String = ""
    var statusTakePhoto: Bool = true
    var statusValidateImage: ValidateCard = .ERROR
    var statusScreen: String = "vertical" //horizontal
    let validateInput = SBKValidateInput.shared
    
    var completionSuccessCard: (_ data: [String: Any]?, _ cardImage: Data?, _ faceImage: Data?) -> Void = {_,_,_ in}
    
    private var previewWidth: CGFloat = 128.0
    private var previewHeight: CGFloat = 128.0
    private var cropZone: CGRect?
    private var cropImage: CGRect?
    var overlayView : OverLayCardView?
    override func viewDidLoad() {
        super.viewDidLoad()
        self.btnRotateScreen.layer.cornerRadius = 20
    }
    
    init() {
        super.init(nibName: "SBKCaptureCardVC", bundle: Bundle(for: SBKCaptureCardVC.self))
    }
    
    required public init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func loadConfigUI() {
        self.lbStep.text = String(format: "Step %d/3", checkScreen)
        self.lbCopyright.textColor = UIColor.colorFromHexa(Global.colorConfig)
        self.lbDescription.textColor = UIColor.colorFromHexa(Global.colorConfig)
        if Global.frontConfig == "" {
            self.lbCopyright.font = UIFont(name: Global.frontConfig, size: 17)
            self.lbDescription.font = UIFont(name: Global.frontConfig, size: 17)
        }
        
        let label = UILabel()
        label.frame = CGRect(x: self.view.center.x - 50, y: self.viewCamera.frame.size.height / 8 + self.viewCamera.frame.origin.y - 40, width: 100, height: 20)
        label.textAlignment = .center
        label.text = self.checkScreen == 1 ? "Use front" : "Use back"
        label.textColor = UIColor.white
        
        self.view.addSubview(label)
    }
    
    //Cài đặt máy ảnh
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        self.loadConfigUI()
        self.lbDescription.text = self.descriptionScreen
        captureSession.sessionPreset = .high
        guard let backCamera = AVCaptureDevice.default(for: AVMediaType.video)
        else {
            print("Unable to access back camera!")
            return
        }
        do {
            let input = try AVCaptureDeviceInput(device: backCamera)
            if captureSession.canAddInput(input) {
                captureSession.addInput(input)
            }
            if captureSession.canAddOutput(stillImageOutput) {
                captureSession.addOutput(stillImageOutput)
                setupLivePreview()
            }
        }
        catch let error  {
            print("Error Unable to initialize back camera:  \(error.localizedDescription)")
        }
        self.getCameraFrames()
        DispatchQueue.global(qos: .userInitiated).async {
            self.captureSession.startRunning()
        }
        DispatchQueue.main.async {
            self.videoPreviewLayer.frame = self.viewCamera.bounds
        }
        
    }
    
    
    //set frame preview
    func setupLivePreview() {
        videoPreviewLayer = AVCaptureVideoPreviewLayer(session: captureSession)
        
        videoPreviewLayer.videoGravity = .resizeAspectFill
        videoPreviewLayer.connection?.videoOrientation = .portrait
        viewCamera.layer.addSublayer(videoPreviewLayer)
        
        if let viewWithTag = self.viewCamera.viewWithTag(11) {
            viewWithTag.removeFromSuperview()
        }
        
        if let viewWithTag = self.viewCamera.viewWithTag(22) {
            viewWithTag.removeFromSuperview()
        }
        initOverLay()
        
    }
    func initOverLay(){
        if self.overlayView == nil {
            let fWidth = self.view.bounds.size.width
            let fHeight = self.view.bounds.size.height
            let squareWidth = fWidth/2
            self.overlayView  = OverLayCardView(frame: self.viewCamera.bounds)
            self.overlayView!.marginTop = self.viewCamera.frame.origin.x + fWidth/20
            previewWidth = self.viewCamera.frame.width * 18 / 20
            previewHeight = self.viewCamera.frame.width * 18 * 3 / 20 / 4
            self.overlayView!.previewWidth = self.previewWidth
            self.overlayView!.previewHeight = self.previewHeight
            self.overlayView!.borderLength = 50
            self.overlayView!.borderWidth = 5
            self.overlayView!.connerRadius = 0
            viewCamera.addSubview(self.overlayView!)
            self.overlayView!.setLayer()
        }
    }
    
    
    //Xử lý data sâu khi chụp
    @available(iOS 11.0, *)
    func photoOutput(_ output: AVCapturePhotoOutput, didFinishProcessingPhoto photo: AVCapturePhoto, error: Error?) {
        
        guard let imageData = photo.fileDataRepresentation()
        else { return }
        
        self.captureSession.stopRunning()
        self.captureSession.removeOutput(self.videoDataOutput)
        self.captureSession.removeOutput(self.stillImageOutput)
        
        let viewController = SBKResultCapture()
        viewController.imageData = imageData
        viewController.checkScreen = self.checkScreen
        viewController.cropZone = self.cropImage!
        viewController.idFront = self.idFront
        viewController.URLToken = self.URLToken
        viewController.statusScreen = self.statusScreen
        viewController.completionSuccessResultCard = { [weak self] data, cardImage, faceImage in
            guard let `self` = self else { return }
            self.completionSuccessCard(data, cardImage, faceImage)
        }
        
        self.navigationController?.pushViewController(viewController, animated: true)
    }
    
    //Sự kiện chụp ảnh
    @IBAction func onCapturePhoto(_ sender: Any) {
        if (self.statusValidateImage == ValidateCard.IMAGE_FRONT && self.checkScreen == 1) || (self.statusValidateImage == .IMAGE_BACK && self.checkScreen == 2) {
            if #available(iOS 11.0, *) {
                let settings = AVCapturePhotoSettings(format: [AVVideoCodecKey: AVVideoCodecType.jpeg])
                stillImageOutput.capturePhoto(with: settings, delegate: self)
            } else {
                let settings = AVCapturePhotoSettings(format: [AVVideoCodecKey: AVVideoCodecJPEG])
                stillImageOutput.capturePhoto(with: settings, delegate: self)
            }
        }
    }
    
    
    @IBAction func onSelectRotateScreen(_ sender: Any) {
    }
    
    @IBAction func onSelectBack(_ sender: Any) {
        if checkScreen == 1 {
            self.navigationController?.popViewController(animated: true)
        } else {
            for controller in self.navigationController!.viewControllers as Array {
                if controller.isKind(of: SBKCaptureCardVC.self) {
                    self.navigationController!.popToViewController(controller, animated: true)
                    break
                }
            }
        }
    }
}

extension SBKCaptureCardVC: AVCaptureVideoDataOutputSampleBufferDelegate {
    private func getCameraFrames() {
        self.videoDataOutput.videoSettings = [(kCVPixelBufferPixelFormatTypeKey as NSString) : NSNumber(value: kCVPixelFormatType_32BGRA)] as [String : Any]
        self.videoDataOutput.alwaysDiscardsLateVideoFrames = true
        self.videoDataOutput.setSampleBufferDelegate(self, queue: DispatchQueue(label: "camera_frame_processing_queue"))
        self.captureSession.addOutput(self.videoDataOutput)
        guard let connection = self.videoDataOutput.connection(with: AVMediaType.video),
              connection.isVideoOrientationSupported else { return }
        connection.videoOrientation = .portrait
    }
    
    func captureOutput( _ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        
        guard let imageFrameInput = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            debugPrint("unable to get image from sample buffer")
            return
        }
        if self.cropZone == nil {
            DispatchQueue.main.async {
                let scaleSizeWidth = self.viewCamera.frame.width / CGFloat( CVPixelBufferGetWidth(imageFrameInput))
                let scaleSizeHeight = self.viewCamera.frame.height / CGFloat( CVPixelBufferGetHeight(imageFrameInput))
                let x = self.overlayView!.marginTop
                let y = (self.viewCamera.frame.height - self.previewHeight) / 2
                self.cropZone = CGRect(x: CGFloat(x) / scaleSizeWidth, y: CGFloat(y + 30) /  scaleSizeHeight, width: CGFloat(self.previewWidth) / scaleSizeWidth , height: CGFloat(self.previewHeight - 60) / scaleSizeHeight )
                self.cropImage = CGRect(x: CGFloat(x - 10) / scaleSizeWidth, y: CGFloat(y + 10) /  scaleSizeHeight, width: CGFloat(self.previewWidth + 20) / scaleSizeWidth , height: CGFloat(self.previewHeight - 20) / scaleSizeHeight )
            }
        }
        
        guard let crop = self.cropZone else {return}
        guard let imageOutput = imageFrameInput.crop(rect: crop, scale: 1.0) else {return}
        let validateImageCard = SBKValidateInput.shared.didOutput(pixelBuffer: imageOutput)
        if self.overlayView != nil {
            DispatchQueue.main.async {
                if let viewWithTag = self.viewCamera.viewWithTag(11) {
                    viewWithTag.removeFromSuperview()
                }
                self.statusValidateImage = validateImageCard
                if validateImageCard == ValidateCard.IMAGE_FAKE || validateImageCard == .ERROR {
                    self.lbDescription.textColor = UIColor.red
                    self.lbDescription.text = "Incorrect card, please check!".localized()
                    self.imgCaution.isHidden = false
                    self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                    self.overlayView!.setBorderColor(color: UIColor.red.cgColor)
                }
                
                if validateImageCard == .IMAGE_FRONT && self.checkScreen == 2 {
                    self.lbDescription.textColor = UIColor.red
                    self.lbDescription.text = "Please put the back of the card in".localized()
                    self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                    self.imgCaution.isHidden = false
                    self.overlayView!.setBorderColor(color: UIColor.red.cgColor)
                }
                
                if validateImageCard == .IMAGE_BACK && self.checkScreen == 1 {
                    self.lbDescription.textColor = UIColor.red
                    self.lbDescription.text = "Please put the front of the card in".localized()
                    self.btnCapture.setImage(UIImage(named: "Button_Do@2x", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                    self.imgCaution.isHidden = false
                    self.overlayView!.setBorderColor(color: UIColor.red.cgColor)
                }
                
                if (self.statusValidateImage == ValidateCard.IMAGE_FRONT && self.checkScreen == 1) || (self.statusValidateImage == .IMAGE_BACK && self.checkScreen == 2) {
                    self.lbDescription.textColor = UIColor.colorFromHexa(Global.colorConfig)
                    self.lbDescription.text = "Are you ready. Let's start!".localized()
                    self.imgCaution.isHidden = true
                    self.btnCapture.setImage(UIImage(named: "iconCap", in: Bundle(for: SBKTutorialVC.self), compatibleWith: nil), for: .normal)
                    self.overlayView!.setBorderColor(color: UIColor.blue.cgColor)
                }
            }
        }
    }
}
